==> udata_ldap-0.1.0.dev3-py2.py3-none-any.whl: udata_ldap/views.py <==
# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import base64
import gssapi

from flask import current_app, request, Response, url_for, redirect
from flask.views import MethodView

from flask_security.utils import login_user

from udata import search, auth, theme
from udata.i18n import I18nBlueprint

from udata.core.user.models import datastore

from .forms import LoginForm
# from .models import UserLdap
from .ldap import manager

from flask_ldap3_login import AuthenticationResponseStatus

bp = I18nBlueprint('ldap', __name__, url_prefix='/ldap',
                   template_folder='templates',)


@bp.before_app_request
def check_remote_user():
    # print('Before request', request.headers)
    remote_user = request.headers.get('REMOTE_USER')
    if not remote_user:
        return
    user = datastore.find_user(ext__remote_user=remote_user)
    if not user:
        # print(fetch_from_ldap(remote_user))
        first_name, last_name = extract_names(remote_user)
        # email, first_name, last_name
        user = datastore.create_user(
            # slug=data['slug'],
            first_name=first_name,
            last_name=last_name,
            email=remote_user,
            active=True,
            ext={'remote_user': remote_user},
            # avatar_url=data['profile'].get('avatar') or None,
            # website=data['profile'].get('website') or None,
            # about=data['profile'].get('about') or None
        )
    # else:
    #     user.first_name = data['first_name']
    #     user.last_name = data['last_name']
    #     user.active = data['is_active']
    #     user.avatar_url = data['profile'].get('avatar') or None
    #     user.website = data['profile'].get('website') or None
    #     user.about = data['profile'].get('about') or None

    # admin_role = datastore.find_or_create_role('admin')
    # if data['is_superuser'] and not user.has_role(admin_role):
    #     datastore.add_role_to_user(user, admin_role)

    # user.save()
    login_user(user)


def redirect_to_login():
    return redirect(url_for('ldap.login'))


@bp.route('/login', endpoint='login')
class LoginView(MethodView):
    def get(self):
        return theme.render('ldap/login.html', form=LoginForm())

    def post(self):
        form = LoginForm(request.form)
        if form.validate():
            username = form.username.data
            password = form.password.data

            result = manager.authenticate(username, password)

            if result.status == AuthenticationResponseStatus.success:
                # data = result.user_info

                user = datastore.find_user(email=result.user_id)
                if not user:
                    user = datastore.create_user(
                        # first_name=data['givenName'][0],
                        # last_name=data['sn'][0],
                        # email=result.user_id,
                        active=True,
                        **manager.extract_user_infos(result.user_info)
                        # avatar_url=data['profile'].get('avatar') or None,
                        # website=data['profile'].get('website') or None,
                        # about=data['profile'].get('about') or None
                    )
                login_user(user)
                next_url = form.next.data or url_for('site.home')
                return redirect(next_url)
        return theme.render('ldap/login.html', form=form)


@bp.route('/negociate')
def negociate():
    if request.headers.get('Authorization', '').startswith('Negotiate '):
        in_token = base64.b64decode(request.headers['Authorization'][10:])
        print('in token', in_token)

        try:
            creds = current_app.extensions['ldap']['creds']
        except KeyError:
            raise RuntimeError('flask-gssapi not configured for this app')

        ctx = gssapi.SecurityContext(creds=creds, usage='accept')

        out_token = ctx.step(in_token)

        if ctx.complete:
            username = str(ctx._inquire(initiator_name=True).initiator_name)
            print('name', username, dir(username))
            # print('attr', username.attributes.items())
            # print('canonical', username.canonicalize())
            # username = str(username)
            # print('attr', username.attributes)
            # print('username', username, str(username), out_token)
            data = manager.get_user_info_from_spnego(username)
            from pprint import pprint
            print('data')
            pprint(data)
            if data:
                user = datastore.find_user(email=data['mail'][0])
                if not user:
                    user = datastore.create_user(
                        # first_name=data['givenName'][0],
                        # last_name=data['sn'][0],
                        # email=data['mail'],
                        active=True,
                        **manager.extract_user_infos(data)
                        # avatar_url=data['profile'].get('avatar') or None,
                        # website=data['profile'].get('website') or None,
                        # about=data['profile'].get('about') or None
                    )
                login_user(user)
                return redirect(url_for('site.home'))

            next_url = request.args['next'] if 'next' in request.args else url_for('site.home')
            # return str(username), out_token
            return redirect(next_url)

    return Response(
        status=401,
        headers={'WWW-Authenticate': 'Negotiate'},
    )

==> udata_ldap-0.2.0-py2.py3-none-any.whl: udata_ldap/views.py <==
# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import base64

from flask import current_app, flash, request, Response, url_for, redirect
from flask.views import MethodView

from flask_security.utils import login_user

from udata import theme
from udata.i18n import I18nBlueprint

from udata.core.user.models import datastore

from .forms import LoginForm
from .ldap import manager

from flask_ldap3_login import AuthenticationResponseStatus

bp = I18nBlueprint('ldap', __name__, url_prefix='/ldap',
                   template_folder='templates',)


@bp.before_app_request
def check_remote_user():
    if not current_app.config.get('LDAP_ALLOW_REMOTE_USER', False):
        return
    remote_user = request.headers.get('REMOTE_USER')
    if not remote_user:
        return
    data = manager.get_trusted_user_infos(remote_user)
    if data:
        user = datastore.find_user(email=data['mail'][0])
        if not user:
            user = datastore.create_user(
                active=True,
                **manager.extract_user_infos(data)
            )
        login_user(user)


def redirect_to_login():
    params = {}
    if 'next' in request.args:
        next_url = request.args['next']
        if next_url.startswith('/') and not next_url.startswith('//'):
            next_url = '{0}://{1}{2}'.format(request.scheme,
                                             current_app.config['SERVER_NAME'],
                                             next_url)
        params['next'] = next_url
    if 'message' in request.args:
        params['message'] = request.args['message']
    return redirect(url_for('ldap.login', **params))


@bp.route('/login', endpoint='login')
class LoginView(MethodView):
    def get(self):
        params = {}
        if 'next' in request.args:
            params['next'] = request.args['next']

        if 'message' in request.args and not request.is_json:
            flash(request.args['message'])

        return theme.render('ldap/login.html', form=LoginForm(**params))

    def post(self):
        form = LoginForm(request.form)
        if form.validate():
            username = form.username.data
            password = form.password.data

            result = manager.authenticate(username, password)

            if result.status == AuthenticationResponseStatus.success:
                user = datastore.find_user(email=result.user_id)
                if not user:
                    user = datastore.create_user(
                        active=True,
                        **manager.extract_user_infos(result.user_info)
                    )
                login_user(user)
                next_url = form.next.data or url_for('site.home')
                return redirect(next_url)
        return theme.render('ldap/login.html', form=form)


@bp.route('/negociate')
def negociate():
    if request.headers.get('Authorization', '').startswith('Negotiate '):
        in_token = base64.b64decode(request.headers['Authorization'][10:])

        ctx = manager.kerberos_security_context

        ctx.step(in_token)

        if ctx.complete:
            username = str(ctx._inquire(initiator_name=True).initiator_name)
            data = manager.get_trusted_user_infos(username)
            if data:
                user = datastore.find_user(email=data['mail'][0])
                if not user:
                    user = datastore.create_user(
                        active=True,
                        **manager.extract_user_infos(data)
                    )
                login_user(user)
                return redirect(url_for('site.home'))

            next_url = request.args['next'] if 'next' in request.args else url_for('site.home')
            return redirect(next_url)

    return Response(
        status=401,
        headers={'WWW-Authenticate': 'Negotiate'},
    )
